@@ -10,7 +10,8 @@ import pprint
 import typing
 from dataclasses import dataclass, field
 from enum import Enum
-from typing import Any, Iterable, Iterator, MutableSet, Sequence
+from types import NoneType
+from typing import AbstractSet, Any, Iterable, Iterator, Sequence
 
 import pymongo.collection
 import pymongo.database
@@ -46,18 +47,6 @@ class CopyBase:
 class DictBase:
     """Base class for serialize objects to dict."""
 
-    @classmethod
-    def _dict_encode_enum(cls, obj) -> Any:
-        """Classmethod to represents enumerates in dictionaries."""
-
-        return obj
-
-    @classmethod
-    def _dict_encode_set(cls, obj) -> Any:
-        """Classmethod to represent sets in dictionaries."""
-
-        return obj
-
     def _dict_repr(self) -> Any:
         """
         Returns the dict representation of your object.
@@ -92,32 +81,39 @@ class DictBase:
                         continue
                     case dict(dict_) if issubclass(type_, DictBase) and not lazy:
                         new_data[k] = decode_dict(type_, dict_)
-                    case [*_] as list_ if not isinstance(list_, set) and type_origin and issubclass(type_origin, Iterable) and issubclass(value_type, DictBase) and not lazy:
+                    case [*_] as list_ if not isinstance(list_, set) and type_origin and type_origin is not typing.Union and issubclass(type_origin, Iterable) and issubclass(value_type, DictBase) and not lazy:
                         new_data[k] = [decode_dict(value_type, dict_) for dict_ in list_]
-                    case bytes(bytes_) if type_ is not bytes:
-                        new_data[k] = pickle.loads(bytes_)
+                    case bytes(bytes_):
+                        try:
+                            new_data[k] = pickle.loads(bytes_)
+                        except (pickle.UnpicklingError, EOFError):
+                            pass
 
             return new_data if issubclass(cls_, dict) else cls_(**new_data)
 
         return decode_dict(cls, data)
 
-    def to_dict(self) -> dict:
+    def to_dict(self, pickle_types: tuple | list = (), recursive=False) -> Any:
         """Returns the representation of the object as a dictionary."""
 
+        def encode_obj(obj_) -> Any:
+            match obj_:
+                case _ if isinstance(obj_, pickle_types):
+                    return pickle.dumps(obj_)
+                case MongoBase():
+                    return obj_.to_dict(pickle_types)
+                case [*_, _] as objs:
+                    return [encode_obj(obj) for obj in objs]
+                case _:
+                    return obj_
+
         if not isinstance(dict_repr := self._dict_repr(), dict):
             return dict_repr
 
+        # noinspection DuplicatedCode
         self_vars = dict_repr.copy()
         for k, v in self_vars.items():
-            match v:
-                case DictBase() as obj if not isinstance(obj, MutableSet):
-                    self_vars[k] = obj.to_dict()
-                case [*_, DictBase()] as objs if not isinstance(objs, MutableSet):
-                    self_vars[k] = [obj.to_dict() for obj in v]
-                case Enum() as obj:
-                    self_vars[k] = self._dict_encode_enum(obj)
-                case obj if isinstance(obj, MutableSet):
-                    self_vars[k] = self._dict_encode_set(obj)
+            self_vars[k] = encode_obj(v)
 
         return self_vars
 
@@ -125,41 +121,6 @@ class DictBase:
 class JSONBASE:
     """Base class for serialize objects to json."""
 
-    @classmethod
-    def _json_decode_dict(cls, cls_: Any, dict_: dict) -> Any:
-        """Classmethod to decode JSON dictionaries to anything."""
-
-        kwargs = {}
-        for k, v in dict_.items():
-            k = cls._json_decode_str(k)
-            v = cls._json_decode_str(v)
-            if isinstance(v, dict):
-                try:
-                    v = cls._json_decode_dict(typing.get_type_hints(cls_)[k], v)
-                except KeyError:
-                    pass
-            kwargs[k] = v
-        return cls_(**kwargs)
-
-    @classmethod
-    def _json_decode_list(cls, obj: Any) -> Any:
-        """Classmethod to decode JSON lists to anything."""
-
-        return [cls._json_decode_str(e) for e in obj]
-
-    @classmethod
-    def _json_decode_str(cls, obj: Any) -> Any:
-        """Classmethod to decode JSON strings to anything."""
-
-        if not isinstance(obj, str):
-            return obj
-        try:
-            bytes_ = base64.b64decode(obj.encode(), validate=True)
-            decoded_obj = pickle.loads(bytes_)
-        except (binascii.Error, pickle.UnpicklingError, EOFError):
-            return obj
-        return decoded_obj
-
     def _json_repr(self) -> Any:
         """
         Returns the JSON representation of your object.
@@ -173,37 +134,72 @@ class JSONBASE:
     def from_json(cls, text: str) -> Any:
         """Classmethod that constructs an object given a JSON string."""
 
+        def decode_str(obj_: Any) -> Any:
+            """Inner function to decode JSON strings to anything."""
+
+            if not isinstance(obj_, str):
+                return obj_
+            try:
+                bytes_ = base64.b64decode(obj_.encode(), validate=True)
+                decoded_obj = pickle.loads(bytes_)
+            except (binascii.Error, pickle.UnpicklingError, EOFError):
+                return obj_
+            return decoded_obj
+
+        def decode_list(obj_: Any) -> Any:
+            """Inner function to decode JSON lists to anything."""
+
+            return [decode_str(e) for e in obj_]
+
+        def decode_dict(cls_: Any, dict_: dict) -> Any:
+            """Inner function to decode JSON dictionaries to anything."""
+
+            kwargs = {}
+            for k, v in dict_.items():
+                k = decode_str(k)
+                v = decode_str(v)
+                if isinstance(v, dict):
+                    try:
+                        v = decode_dict(typing.get_type_hints(cls_)[k], v)
+                    except KeyError:
+                        pass
+                kwargs[k] = v
+            return cls_(**kwargs)
+
         if not isinstance(text, str):
             raise TypeError(f'must be str, not {type(text).__name__}')
 
         obj = json.loads(text)
         if isinstance(obj, str):
-            return cls._json_decode_str(obj)
+            return decode_str(obj)
         elif isinstance(obj, list):
-            return cls._json_decode_list(obj)
+            return decode_list(obj)
         elif isinstance(obj, dict):
-            return cls._json_decode_dict(cls, obj)
+            return decode_dict(cls, obj)
         else:
             return obj
 
-    def to_json(self, indent: int = None) -> str:
+    def to_json(self, pickle_types: tuple | list = (Enum,), indent: int = None) -> str:
         """Returns the representation of the object as a JSON string."""
 
+        # noinspection PyProtectedMember,PyUnresolvedReferences
         def json_encoder(obj: Any) -> Any:
             if isinstance(obj, bytes):
                 return base64.b64encode(obj).decode()
 
             match obj:
-                case set():
+                case obj if isinstance(obj, AbstractSet):
                     return list(obj)
                 case JSONBASE():
-                    # noinspection PyProtectedMember
-                    return obj._json_repr()
+                    if isinstance(obj, pickle_types) and not obj._json_repr.__qualname__.startswith(obj.__class__.__name__):
+                        return pickle.dumps(obj)
+                    else:
+                        return obj._json_repr()
                 case (datetime.date() | datetime.datetime()) as date_time:
                     return str(date_time)
                 case _:
                     try:
-                        return json.dumps(obj)
+                        return json.dumps(obj, indent=indent)
                     except TypeError:
                         return repr(obj)
 
@@ -234,6 +230,7 @@ class MeanBase:
         elif len(ratios) != len(objects):
             raise ValueError('Wrong ratios length')
 
+        # ----- updates the ratios depending on empty attributes -----
         attributes_ratios = {}
         attributes_ratios_length = {}
         for attribute_name in attribute_names:
@@ -246,10 +243,10 @@ class MeanBase:
                         ratio_part_to_add = attributes_ratios[attribute_name][object_index] / attributes_ratios_length[attribute_name]
                     except ZeroDivisionError:
                         ratio_part_to_add = 0
-                    for ratio_index, _ in enumerate(attributes_ratios[attribute_name]):
-                        if ratio_index != object_index:
-                            attributes_ratios[attribute_name][ratio_index] += ratio_part_to_add
                     attributes_ratios[attribute_name][object_index] = 0
+                    for ratio_index, _ in enumerate(attributes_ratios[attribute_name]):
+                        if attributes_ratios[attribute_name][ratio_index]:
+                            attributes_ratios[attribute_name][ratio_index] += ratio_part_to_add
 
         attribute_values = {}
         timezone: datetime.timezone | None = None
@@ -273,7 +270,7 @@ class MeanBase:
         return cls(**attribute_values)
 
 
-class _MongoBase(DictBase, BytesBase):
+class MongoBase(DictBase, BytesBase):
     """
     Base class for mapping objects to mongo documents and vice versa (Object Document Mapper).
 
@@ -308,7 +305,7 @@ class _MongoBase(DictBase, BytesBase):
         self._create_unique_indices()
 
     def __post_init__(self):
-        _MongoBase.__init__(self)
+        MongoBase.__init__(self)
 
     def __bytes__(self):
         return pickle.dumps(self.to_dict())
@@ -337,9 +334,9 @@ class _MongoBase(DictBase, BytesBase):
             return value
 
         match value:
-            case ObjectId() as object_id if issubclass(type_, _MongoBase):
+            case ObjectId() as object_id if issubclass(type_, MongoBase):
                 value = type_.find_one({'_id': object_id})
-            case [*_, ObjectId()] as object_ids if type_arg := iterables.find(typing.get_args(type_), _MongoBase):
+            case [*_, ObjectId()] as object_ids if type_arg := iterables.find(typing.get_args(type_), MongoBase):
                 value = [result for object_id in object_ids if (result := type_arg.find_one({'_id': object_id}))]
             case _:
                 return value
@@ -347,18 +344,12 @@ class _MongoBase(DictBase, BytesBase):
         super().__setattr__(attribute_name, value)
         return value
 
-    def __getstate__(self):
-        return self.to_dict()
-
     def __hash__(self):
         if unique_attributes := self.unique_attributes:
             return hash(tuple(unique_attributes.values()))
         else:
             return hash(self._id)
 
-    def __setstate__(self, state):
-        self.__dict__ = vars(self.from_dict(state, lazy=False))
-
     def _create_unique_indices(self):
         """Create the unique indices in the database based on _unique_keys and _nullable_unique_keys attributes."""
 
@@ -381,24 +372,16 @@ class _MongoBase(DictBase, BytesBase):
 
         return {k: v for k, v in self_vars.items() if k not in ('_database', '_unique_keys', '_nullable_unique_keys', 'collection')}
 
-    @classmethod
-    def _mongo_encode_enum(cls, obj) -> Any:
-        return pickle.dumps(obj)
-
-    @classmethod
-    def _mongo_encode_set(cls, obj) -> Any:
-        return pickle.dumps(obj)
-
     def _mongo_repr(self) -> Any:
         """Returns the object representation to save in mongo database."""
 
-        return self._dict_repr()
+        return {k: v.value if isinstance(v, Enum) else v for k, v in self._dict_repr().items()}
 
     def delete(self, cascade=False):
         """
         Delete the object from the database.
 
-        If cascade=True all objects whose classes inherit from _MongoBase are also deleted.
+        If cascade=True all objects whose classes inherit from MongoBase are also deleted.
         """
 
         if cascade:
@@ -433,7 +416,7 @@ class _MongoBase(DictBase, BytesBase):
             return [cls.from_dict(document) for document in cursor]
 
     @classmethod
-    def find_one(cls, query: dict = None, sort_keys: str | Iterable[str | tuple[str, int]] = ()) -> _MongoBase | None:
+    def find_one(cls, query: dict = None, sort_keys: str | Iterable[str | tuple[str, int]] = ()) -> MongoBase | None:
         """Query the collection and return the first match."""
 
         return next(cls.find(query, sort_keys, lazy=True), None)
@@ -448,16 +431,16 @@ class _MongoBase(DictBase, BytesBase):
     def from_bytes(cls, bytes_: bytes) -> Any:
         return cls.from_dict(super().from_bytes(bytes_))
 
-    def get_referenced_objects(self) -> list[_MongoBase]:
-        """Returns all referenced objects whose classes inherit from _MongoBase."""
+    def get_referenced_objects(self) -> list[MongoBase]:
+        """Returns all referenced objects whose classes inherit from MongoBase."""
 
         referenced_objects = []
         for k, v in vars(self).items():
             match v:
-                case _MongoBase() as obj:
+                case MongoBase() as obj:
                     referenced_objects.append(obj)
-                case [*_, _MongoBase()] as objs:
-                    referenced_objects.extend(obj for obj in objs if isinstance(obj, _MongoBase))
+                case [*_, MongoBase()] as objs:
+                    referenced_objects.extend(obj for obj in objs if isinstance(obj, MongoBase))
 
         return referenced_objects
 
@@ -465,7 +448,7 @@ class _MongoBase(DictBase, BytesBase):
     def object_id(self):
         return self._id
 
-    def pull_from_database(self, query: dict = None, exclude: Iterable[str] = (), database_priority=False):
+    def pull_from_database(self, overwrite_fields: Iterable[str] = ('_id',), exclude: Iterable[str] = ()):
         """
         Updates the values of the current object with the values of the same object located in the database.
 
@@ -475,46 +458,61 @@ class _MongoBase(DictBase, BytesBase):
         Ignore the attributes specified in exclude.
         """
 
-        if not query:
-            unique_attributes = self.unique_attributes
+        unique_attributes = self.unique_attributes
 
-            if all(unique_attributes.values()):
-                query = {}
-                for k, v in unique_attributes.items():
-                    if isinstance(v, _MongoBase):
-                        v.pull_from_database(exclude=exclude, database_priority=database_priority)
-                        v = v._id
-                    query[k] = v
-            else:
-                query = {'_id': self._id}
+        if any(value is None for value in unique_attributes.values()):
+            query = {'_id': self._id}
+        else:
+            query = {}
+            for k, v in unique_attributes.items():
+                if isinstance(v, MongoBase):
+                    v.pull_from_database(overwrite_fields, exclude)
+                    v = v._id
+                query[k] = v
 
         if document := self.collection.find_one(query):
             for database_key, database_value in vars(self.from_dict(document)).items():
-                if database_key in exclude:
-                    continue
-
                 self_value = getattr(self, database_key)
-                if self_value is None or (database_priority and database_value is not None) or database_key == '_id':
+                if (
+                        database_key not in exclude
+                        and
+                        (
+                                database_key in overwrite_fields and database_value is not None
+                                or
+                                self_value is None
+                                or
+                                isinstance(self_value, Iterable) and not self_value
+                        )
+                ):
                     super().__setattr__(database_key, database_value)
 
     def resolve(self):
-        """Resolve all the ObjectId references (ObjectId -> _MongoBase)."""
+        """Resolve all the ObjectId references (ObjectId -> MongoBase)."""
 
         for k in vars(self):
             getattr(self, k)
 
-    def save(self, pull_exclude: Iterable[str] = (), pull_database_priority=False, references=True):
+    def save(
+        self,
+        pickle_types: tuple | list = (AbstractSet,),
+        references=True,
+        pull_overwrite_fields: Iterable[str] = ('_id',),
+        pull_exclude: Iterable[str] = ()
+    ):
         """
         Save (insert or update) the current object in the database.
 
-        If references=True it saves the objects without redundancy (_MongoBase -> ObjectId).
+        If references=True it saves the objects without redundancy (MongoBase -> ObjectId).
         """
 
-        self.pull_from_database(exclude=pull_exclude, database_priority=pull_database_priority)
-        for referenced_object in self.get_referenced_objects():
-            referenced_object.save(pull_exclude, pull_database_priority, references)
+        if self.collection is None:
+            return
 
-        data = self.to_mongo()
+        self.pull_from_database(pull_overwrite_fields, pull_exclude)
+        for referenced_object in self.get_referenced_objects():
+            referenced_object.save(pickle_types, references, pull_overwrite_fields, pull_exclude)
+
+        data = self.to_mongo(pickle_types)
 
         if references:
             for k, v in data.items():
@@ -526,23 +524,27 @@ class _MongoBase(DictBase, BytesBase):
 
         self.collection.find_one_and_update({'_id': self._id}, {'$set': data}, upsert=True)
 
-    def to_mongo(self) -> dict:
+    def to_mongo(self, pickle_types: tuple | list = (AbstractSet,)) -> Any:
         """Returns the representation of the object as a mongo compatible dictionary."""
 
-        if not isinstance(dict_repr := self._mongo_repr(), dict):
-            return dict_repr
+        def encode_obj(obj_) -> Any:
+            match obj_:
+                case MongoBase() if not isinstance(obj_, pickle_types):
+                    return obj_.to_mongo(pickle_types)
+                case [*_, _] as objs:
+                    return [encode_obj(obj) for obj in objs]
+                case _ if not isinstance(obj_, (NoneType, int, float, str, bool, bytes, Sequence, dict, datetime.date, datetime.datetime, ObjectId)):
+                    return pickle.dumps(obj_)
+                case _:
+                    return obj_
 
-        self_vars = dict_repr.copy()
+        if not isinstance(mongo_repr := self._mongo_repr(), dict):
+            return mongo_repr
+
+        # noinspection DuplicatedCode
+        self_vars = mongo_repr.copy()
         for k, v in self_vars.items():
-            match v:
-                case MongoBase() as obj if not isinstance(obj, MutableSet):
-                    self_vars[k] = obj.to_mongo()
-                case [*_, MongoBase()] as objs if not isinstance(objs, MutableSet):
-                    self_vars[k] = [obj.to_mongo() for obj in v]
-                case Enum() as obj:
-                    self_vars[k] = self._mongo_encode_enum(obj)
-                case obj if isinstance(obj, MutableSet):
-                    self_vars[k] = self._mongo_encode_set(obj)
+            self_vars[k] = encode_obj(v)
 
         return self_vars
 
@@ -553,18 +555,23 @@ class _MongoBase(DictBase, BytesBase):
         values.
         """
 
-        return {unique_key: getattr(self, unique_key, None) for unique_key in self._unique_keys}
+        unique_attributes = {}
+        for unique_key in self._unique_keys:
+            attribute_value = getattr(self, unique_key, None)
+            unique_attributes[unique_key] = attribute_value.value if isinstance(attribute_value, Enum) else attribute_value
+
+        return unique_attributes
 
 
 @dataclass(eq=False)
-class MongoBase(_MongoBase):
+class DCMongoBase(MongoBase):
     """Base class to be inherited by an unfrozen dataclass."""
 
     _id: ObjectId = field(kw_only=True, default_factory=ObjectId)
 
 
 @dataclass(eq=False, frozen=True)
-class FrozenMongoBase(_MongoBase):
+class FrozenDCMongoBase(MongoBase):
     """Base class to be inherited by a frozen dataclass."""
 
     _id: ObjectId = field(kw_only=True, default_factory=ObjectId)
@@ -597,9 +604,6 @@ class FlanaEnum(JSONBASE, DictBase, CopyBase, BytesBase, Enum):
     def _json_repr(self) -> Any:
         return bytes(self)
 
-    def _mongo_repr(self) -> Any:
-        return pickle.dumps(self)
-
     @classmethod
     @property
     def items(cls) -> list[tuple[str, int]]:
